@@ -110,7 +110,7 @@ namespace
         };
     }
 
-    std::vector<PackageDependInfo> extract_depend_info(const std::vector<const InstallPlanAction*>& install_actions,
+    std::vector<PackageDependInfo> extract_depend_info(const std::vector<InstallPlanAction>& install_actions,
                                                        const Triplet& default_triplet,
                                                        const Triplet& host_triplet,
                                                        const int max_depth)
@@ -126,10 +126,8 @@ namespace
         out.reserve(install_actions.size());
 
         std::map<std::string, PackageDependInfo&> package_dependencies;
-        for (const InstallPlanAction* pia : install_actions)
+        for (const InstallPlanAction& install_action : install_actions)
         {
-            const InstallPlanAction& install_action = *pia;
-
             const std::vector<std::string> dependencies =
                 Util::fmap(install_action.package_dependencies,
                            [&decorated_name](const PackageSpec& spec) { return decorated_name(spec); });
@@ -426,20 +424,13 @@ namespace vcpkg
             {nullptr, host_triplet, UnsupportedPortAction::Warn, UseHeadVersion::No, Editable::No});
         action_plan.print_unsupported_warnings();
 
-        if (!action_plan.remove_actions.empty())
+        if (!action_plan.already_installed.empty() || !action_plan.remove_actions.empty())
         {
             Checks::unreachable(VCPKG_LINE_INFO, "Only install actions should exist in the plan");
         }
 
-        std::vector<const InstallPlanAction*> install_actions =
-            Util::fmap(action_plan.already_installed, [&](const auto& action) { return &action; });
-        for (auto&& action : action_plan.install_actions)
-        {
-            install_actions.push_back(&action);
-        }
-
         std::vector<PackageDependInfo> depend_info =
-            extract_depend_info(install_actions, default_triplet, host_triplet, strategy.max_depth);
+            extract_depend_info(action_plan.install_actions, default_triplet, host_triplet, strategy.max_depth);
 
         if (strategy.format == DependInfoFormat::Dot)
         {
