@@ -1,9 +1,9 @@
-#include <vcpkg/commands.z-upload-metrics.h>
-
-#if defined(_WIN32)
 #include <vcpkg/base/checks.h>
+#include <vcpkg/base/contractual-constants.h>
 #include <vcpkg/base/files.h>
+#include <vcpkg/base/system.debug.h>
 
+#include <vcpkg/commands.z-upload-metrics.h>
 #include <vcpkg/metrics.h>
 #include <vcpkg/vcpkgcmdarguments.h>
 
@@ -26,8 +26,18 @@ namespace vcpkg
         const auto parsed = args.parse_arguments(CommandZUploadMetricsMetadata);
         const auto& payload_path = parsed.command_arguments[0];
         auto payload = fs.read_contents(payload_path, VCPKG_LINE_INFO);
-        winhttp_upload_metrics(payload);
+        if (!curl_upload_metrics(payload))
+        {
+            Debug::println("Failed to upload metrics");
+            Checks::exit_fail(VCPKG_LINE_INFO);
+        }
+
+        std::error_code ec;
+        fs.remove(payload_path, ec);
+        if (ec)
+        {
+            Debug::println("Failed to remove file after upload: {}", ec.message());
+        }
         Checks::exit_success(VCPKG_LINE_INFO);
     }
 }
-#endif // defined(_WIN32)
