@@ -7,6 +7,7 @@
 
 #include <vcpkg/documentation.h>
 #include <vcpkg/packagespec.h>
+#include <vcpkg/versions.h>
 
 namespace
 {
@@ -154,6 +155,19 @@ namespace vcpkg
         }
     }
 
+    std::string format_full_version_spec(const PackageSpec& spec,
+                                         const InternalFeatureSet& feature_list,
+                                         const Version& version)
+    {
+        if (feature_list.empty_or_only_core())
+        {
+            return fmt::format("{}@{}", spec.to_string(), version);
+        }
+
+        const std::string features = Strings::join(",", feature_list);
+        return fmt::format("{}[{}]:{}@{}", spec.name(), features, spec.triplet(), version);
+    }
+
     const std::string& PackageSpec::name() const { return this->m_name; }
 
     Triplet PackageSpec::triplet() const { return this->m_triplet; }
